package de.ozgcloud.processor.vorgang;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;

import de.itvsh.kop.common.binaryfile.FileId;
import de.ozgcloud.apilib.file.OzgCloudFileGroup;
import de.ozgcloud.apilib.file.OzgCloudFileId;
import de.ozgcloud.apilib.vorgang.OzgCloudAntragsteller;
import de.ozgcloud.apilib.vorgang.OzgCloudEingang;
import de.ozgcloud.apilib.vorgang.OzgCloudEingangHeader;
import de.ozgcloud.apilib.vorgang.OzgCloudFormDataEntry;
import de.ozgcloud.apilib.vorgang.OzgCloudFormField;
import de.ozgcloud.apilib.vorgang.OzgCloudSubForm;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgang;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;

@Mapper
public interface ProcessorVorgangMapper {

	@Mapping(target = "eingang", qualifiedByName = "mapEingang", source = "eingangs")
	@Mapping(target = "formId", qualifiedByName = "getFormId", source = "eingangs")
	@Mapping(target = "formEngineName", qualifiedByName = "mapFormEngineName", source = "eingangs")
	Vorgang mapVorgang(OzgCloudVorgang ozgCloudVorgang);

	default VorgangId mapId(OzgCloudVorgangId id) {
		return VorgangId.from(id);
	}

	@Named("getFormId")
	default String getFormId(List<OzgCloudEingang> eingang) {
		if (CollectionUtils.isEmpty(eingang)) {
			return null;
		}
		return eingang.get(0).getHeader().getFormId();
	}

	@Named("mapEingang")
	default Eingang selectFirst(List<OzgCloudEingang> eingang) {
		if (!eingang.isEmpty()) {
			return mapEingang(eingang.get(0));
		}
		return null;
	}

	@Named("mapFormEngineName")
	default String mapFormEngineName(List<OzgCloudEingang> eingangs) {
		return eingangs.stream().findFirst().map(OzgCloudEingang::getHeader)
				.map(OzgCloudEingangHeader::getFormEngineName).orElse(StringUtils.EMPTY);
	}

	@Mapping(target = "attachment", ignore = true)
	@Mapping(target = "formData", source = "formDataEntrys")
	Eingang mapEingang(OzgCloudEingang eingang);

	List<FormDataEntry> map(List<OzgCloudFormDataEntry> value);

	default FormDataEntry mapEntry(OzgCloudFormDataEntry entry) {
		if (entry.isFormEntry()) {
			return mapFormField((OzgCloudFormField) entry);
		} else {
			return mapSubForm((OzgCloudSubForm) entry);
		}
	}

	FormField mapFormField(OzgCloudFormField formField);

	@Mapping(target = "formItem", ignore = true)
	@Mapping(target = "formItems", source = "formEntrys")
	SubForm mapSubForm(OzgCloudSubForm subForm);

	@Mapping(target = "file", ignore = true)
	FileGroup mapFileGroup(OzgCloudFileGroup fileGroup);

	Antragsteller mapAntragsteller(OzgCloudAntragsteller antragsteller);

	default FileId fromOzgCloudId(OzgCloudFileId ozgCloudFileId) {
		return FileId.from(ozgCloudFileId.toString());
	}
}
