package de.ozgcloud.processor.vorgang;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import de.itvsh.ozg.pluto.command.VorgangCreatedEvent;
import de.ozgcloud.processor.processor.ProcessorService;
import de.ozgcloud.processor.result.ResultService;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Component("processorVorgangEventListener")
@ConditionalOnProperty(prefix = "ozgcloud.processors[0]", name = "address")
@RequiredArgsConstructor
@Log4j2
class VorgangEventListener {

	private final VorgangService service;
	private final ProcessorService processorService;
	private final ResultService resultService;

	@EventListener
	public void triggerNewVorgangProcessors(VorgangCreatedEvent event) {
		try {
			var vorgang = service.getVorgang(VorgangId.from(event.getSource()));
			processorService.processVorgang(vorgang).forEach(processorResultMono -> processorResultMono
					.doOnError(cause -> handleError(cause, event.getSource())).onErrorComplete()
					.map(result -> result.toBuilder().vorgangId(VorgangId.from(event.getSource())).build())
					.subscribe(resultService::processResult));
		} catch (RuntimeException e) {
			handleError(e, event.getSource());
		}
	}

	private void handleError(Throwable cause, String vorgangId) {
		LOG.error("Error on procession Vorgang {} externally", vorgangId, cause);
		resultService.processError(cause, vorgangId);
	}
}
