package de.ozgcloud.processor.processor;

import static org.assertj.core.api.Assertions.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.Spy;

import de.ozgcloud.processor.processor.ProcessorProperties.Processor;
import de.ozgcloud.processor.vorgang.VorgangTestFactory;

class ProcessorServiceTest {


	@Spy
	@InjectMocks
	private ProcessorService service;

	@Mock
	private ProcessorProperties properties;
	@Mock
	private ProcessorConfiguration processorConfiguration;


	@Nested
	class TestProcessVorgang {

		@Nested
		class GetProcessorByFormId {

			private Processor configuredProcessor = ProcessorTestFactory.create();

			@BeforeEach
			void setup() {
				Mockito.when(properties.getProcessors()).thenReturn(List.of(configuredProcessor));
			}

			@Test
			void shouldReturnProcessor() {
				var processor = service.getProcessors(VorgangTestFactory.create());

				assertThat(processor).contains(configuredProcessor);
			}

			@Test
			@DisplayName("should return empty if formId is not found")
			void shouldReturnEmptyIfNoFormId() {
				var vorgang = VorgangTestFactory.createBuilder().formId("other").build();

				var processor = service.getProcessors(vorgang);

				assertThat(processor).isEmpty();
			}

			@Test
			@DisplayName("should return empty if formEngineName is not found")
			void shouldReturnEmptyIfNoFormEngineName() {
				var vorgang = VorgangTestFactory.createBuilder().formEngineName("other").build();

				var processor = service.getProcessors(vorgang);

				assertThat(processor).isEmpty();
			}
		}
	}

}
